@@ -79,67 +79,35 @@ get_var_names <- function(formula, data){
   return(var_names)
 }
 
-get_importance<- function(model, n_permute= NULL, data = NULL){
-
-  if(is.null(n_permute)) n_permute <- ceiling(sqrt(nrow(model$data$data))*3)
-  model<- check_model(model)
-
-  if(inherits(model$loss$call, "character")) {
-    if(!any(model$loss$call  == c("softmax","mse", "mae"))){ return(0)}
-  } else {
-    if(!any(model$loss$call$family  == c("binomial")  )){ return(0)}
-  }
-  loss<- model$loss$loss
-
-  true = model$data$Y
-
-  if(inherits(model$loss$call, "character")) {
-    true = torch::torch_tensor(model$data$Y)
-  } else {
-    if(model$loss$call$family  == c("binomial") ){
-      mode(true) = "numeric"
-      true = torch::torch_tensor(true)
-      }
-  }
-  org_err <- as.numeric(loss( pred = torch::torch_tensor(stats::predict(model,model$data$data, type = "link")) ,true = true)$mean())
-
-
-  importance <- data.frame(variable = get_var_names(model$training_properties$formula, model$data$data[1,]),
-                          importance = c(0))
-
-  for(i in seq_len(nrow(importance))){
-
-    new_err <-c()
-    if(n_permute < ((nrow(model$data$data)**2)-1)){
-      for(k in seq_len(n_permute)){
-
-        perm_preds <- c()
-
-        perm_data <- model$data$data
-        perm_data[, importance$variable[i]] <- perm_data[sample.int(n = nrow(perm_data),replace = FALSE),importance$variable[i]]
-
-        perm_preds <- rbind(perm_preds, stats::predict(model, perm_data, type = "link"))
-
-
-        new_err <- append(new_err, as.numeric(loss(pred = torch::torch_tensor(perm_preds),
-                                   true = true)$mean() ))
-
-
-      }
-    }else{
-      for(j in seq_len(nrow(model$data$data))){
-        perm_data <- model$data$data[j,]
-        for(k in seq_len(nrow(model$data$data))[-j]){
-          perm_data[i] <- model$data$data[k,i]
-          perm_preds <- rbind(perm_preds, stats::predict(model, perm_data, type = "link"))
-          true <- append(true, model$data$Y[j])
-        }
-      }
+check_device = function(device) {
+  if(device == "cuda"){
+    if (torch::cuda_is_available()) {
+      device <- torch::torch_device("cuda")}
+    else{
+      warning("No Cuda device detected, device is set to cpu")
+      device <- torch::torch_device("cpu")
     }
 
-    importance$importance[i] <- mean(new_err)/org_err
-
+  } else if(device == "mps") {
+    if (torch::backends_mps_is_available()) {
+      device <- torch::torch_device("mps")}
+    else{
+      warning("No mps device detected, device is set to cpu")
+      device <- torch::torch_device("cpu")
+    }
+  } else {
+    if(device != "cpu") warning(paste0("device ",device," not know, device is set to cpu"))
+    device <- torch::torch_device("cpu")
   }
+  return(device)
+}
 
-  return(importance)
+
+# taken and adopted from lme4:::RHSForm
+LHSForm = function (form, as.form = FALSE)
+{
+  rhsf <- form[[2]]
+  if (as.form)
+    stats::reformulate(deparse(rhsf))
+  else rhsf
 }
