@@ -2,16 +2,16 @@ source("utils.R")
 
 wrap_dnn = function(pars) {
   testthat::expect_error({model = do.call(dnn, pars)}, NA)
-  testthat::expect_error({predict(model, newdata=pars$X)}, NA)
-  testthat::expect_error({predict(model)}, NA)
-  testthat::expect_error({predict(model, type = "response")}, NA)
-  testthat::expect_error({print(model)}, NA)
-  testthat::expect_error({coef(model)}, NA)
-  testthat::expect_error({plot(model)}, NA)
-  testthat::expect_error({residuals(model)}, NA)
-  testthat::expect_error({summary(model)}, NA)
-  testthat::expect_error({PDP(model, variable = "X.1")}, NA)
-  testthat::expect_error({ALE(model, variable = "X.1")}, NA)
+  testthat::expect_error({.n = predict(model, newdata=pars$X)}, NA)
+  testthat::expect_error({.n = continue_training(model, epochs = 2L, verbose = FALSE)}, NA)
+  testthat::expect_error({.n = predict(model)}, NA)
+  testthat::expect_error({.n = predict(model, type = "response")}, NA)
+  testthat::expect_error({.n = coef(model)}, NA)
+  testthat::expect_error({.n = plot(model)}, NA)
+  testthat::expect_error({.n = residuals(model)}, NA)
+  testthat::expect_error({.n = summary(model)}, NA)
+  suppressWarnings(testthat::expect_error({.n = PDP(model, variable = "X.1")}, NA))
+  suppressWarnings(testthat::expect_error({.n = ALE(model, variable = "X.1")}, NA))
 }
 
 
@@ -38,7 +38,7 @@ testthat::test_that("DNN architecture", {
   skip_if_no_torch()
 
   for(i in 1:length(scenarios)) {
-    wrap_dnn(scenarios[[i]])
+    .n = wrap_dnn(scenarios[[i]])
   }
 })
 
@@ -49,7 +49,9 @@ data = data.frame(Y = Y, X = X)
 scenarios =
   list(
     list(formula = stats::as.formula("Y ~ ."), plot=FALSE, verbose = FALSE, data = data, loss = "softmax", epochs = 1L),
-    list(formula = stats::as.formula("Y ~ ."), plot=FALSE, verbose = FALSE, data = data, loss = stats::binomial(), epochs = 1L)
+    list(formula = stats::as.formula("Y ~ ."), plot=FALSE, verbose = FALSE, data = data, loss = stats::binomial(), epochs = 1L),
+    list(formula = stats::as.formula("Y ~ ."), plot=FALSE, verbose = FALSE, data = data, loss = "softmax", epochs = 1L, bootstrap = 2L),
+    list(formula = stats::as.formula("Y ~ ."), plot=FALSE, verbose = FALSE, data = data, loss = stats::binomial(), epochs = 1L, bootstrap = 2L)
   )
 testthat::test_that("DNN softmax/binomial", {
   testthat::skip_on_cran()
@@ -70,7 +72,12 @@ scenarios =
     list(formula = stats::as.formula("Y.1 ~ ."), plot=FALSE,data = data, loss = stats::gaussian(), epochs = 1L),
     list(formula = stats::as.formula("cbind(Y.1, Y.2, Y.3) ~ ."), plot=FALSE, verbose = FALSE, data = data, loss = stats::gaussian(), epochs = 1L),
     list(formula = stats::as.formula("Y.1 ~ ."), plot=FALSE,data = data, loss = stats::poisson(), epochs = 1L),
-    list(formula = stats::as.formula("cbind(Y.1, Y.2, Y.3) ~ ."), plot=FALSE, verbose = FALSE, data = data, loss = stats::poisson(), epochs = 1L)
+    list(formula = stats::as.formula("cbind(Y.1, Y.2, Y.3) ~ ."), plot=FALSE, verbose = FALSE, data = data, loss = stats::poisson(), epochs = 1L),
+
+    list(formula = stats::as.formula("Y.1 ~ ."), plot=FALSE,data = data, loss = stats::gaussian(), epochs = 1L, bootstrap = 2L),
+    list(formula = stats::as.formula("cbind(Y.1, Y.2, Y.3) ~ ."), plot=FALSE, verbose = FALSE, data = data, loss = stats::gaussian(), epochs = 1L, bootstrap = 2L),
+    list(formula = stats::as.formula("Y.1 ~ ."), plot=FALSE,data = data, loss = stats::poisson(), epochs = 1L, bootstrap = 2L),
+    list(formula = stats::as.formula("cbind(Y.1, Y.2, Y.3) ~ ."), plot=FALSE, verbose = FALSE, data = data, loss = stats::poisson(), epochs = 1L, bootstrap = 2L)
   )
 testthat::test_that("DNN rnorm/poisson", {
   testthat::skip_on_cran()
@@ -83,6 +90,27 @@ testthat::test_that("DNN rnorm/poisson", {
 })
 
 
+X = matrix(runif(3*50), 50, 3)
+Y = matrix(rpois(50*3,lambda = 2), 50, 3)
+data = data.frame(Y = Y, X = X)
+scenarios =
+  list(
+    list(formula = stats::as.formula("Y.1 ~ ."), plot=FALSE,data = data, loss = "mae", epochs = 1L),
+    list(formula = stats::as.formula("cbind(Y.1, Y.2, Y.3) ~ ."), plot=FALSE, verbose = FALSE, data = data, loss = "mae", epochs = 1L),
+    list(formula = stats::as.formula("Y.1 ~ ."), plot=FALSE,data = data, loss = "mae", epochs = 1L, bootstrap = 2L),
+    list(formula = stats::as.formula("cbind(Y.1, Y.2, Y.3) ~ ."), plot=FALSE, verbose = FALSE, data = data, loss = "mae", epochs = 1L, bootstrap = 2L)
+  )
+testthat::test_that("DNN mae", {
+  testthat::skip_on_cran()
+  testthat::skip_on_ci()
+  skip_if_no_torch()
+
+  for(i in 1:length(scenarios)) {
+    wrap_dnn(scenarios[[i]])
+  }
+})
+
+
 testthat::test_that("DNN save and reload", {
   testthat::skip_on_cran()
   testthat::skip_on_ci()
@@ -93,12 +121,12 @@ testthat::test_that("DNN save and reload", {
   validation_set<- sample(c(1:nrow(datasets::iris)),25)
 
   # Build and train  Network
-  nn.fit<- dnn(Sepal.Length~., data = datasets::iris[-validation_set,], epochs = 5L)
+  nn.fit<- dnn(Sepal.Length~., data = datasets::iris[-validation_set,], epochs = 5L, verbose = FALSE, plot = FALSE)
   saveRDS(nn.fit, "test_model.RDS")
   nn.fit = readRDS("test_model.RDS")
-  testthat::expect_error(predict(nn.fit), NA)
-  testthat::expect_error(predict(nn.fit, newdata = datasets::iris[validation_set,]), NA)
-  testthat::expect_error(continue_training(nn.fit,epochs = 5), NA)
+  testthat::expect_error({.n = predict(nn.fit)}, NA)
+  testthat::expect_error({.n = predict(nn.fit, newdata = datasets::iris[validation_set,])}, NA)
+  testthat::expect_error({.n = continue_training(nn.fit,epochs = 5)}, NA)
   file.remove("test_model.RDS")
 })
 
@@ -125,6 +153,8 @@ testthat::expect_error({
                data = datasets::iris[],
                loss = custom_loss,
                epochs = 2L,
+               verbose = FALSE,
+               plot = FALSE,
                custom_parameters = list(scale = 1.0)
   )
   }, NA)
@@ -149,8 +179,10 @@ testthat::expect_error({
 nn.fit<- dnn(cbind(Sepal.Length, Sepal.Width, Petal.Length)~.,
              data = datasets::iris,
              lr = 0.01,
-             epochs = 2L,
+             epochs = 200L,
              loss = custom_loss_MVN,
+             verbose = FALSE,
+             plot = FALSE,
              custom_parameters =
                list(SigmaDiag =  rep(1., 3),
                     SigmaPar = matrix(rnorm(6, sd = 0.001), 3, 2))
@@ -163,7 +195,6 @@ nn.fit<- dnn(cbind(Sepal.Length, Sepal.Width, Petal.Length)~.,
 
 testthat::test_that("DNN coef accuracy check",{
 
-
   #testthat::skip_on_cran()
   #testthat::skip_on_ci()
   skip_if_no_torch()
@@ -172,7 +203,7 @@ testthat::test_that("DNN coef accuracy check",{
   coefs <- runif(n=10)
   data$Y <- apply(data,1,function(x) sum(x*coefs))
 
-  nn.fit<- dnn(Y~., data=data, hidden= NULL,epochs=200)
+  nn.fit<- dnn(Y~., data=data, hidden= NULL,epochs=200, verbose = FALSE, plot = FALSE)
 
   testthat::expect_lt(max(abs((unlist(coef(nn.fit))[-1] - coefs))), 1e02)
 
@@ -180,6 +211,37 @@ testthat::test_that("DNN coef accuracy check",{
 
 
 
+testthat::test_that("DNN baseline loss check",{
+
+  testthat::skip_on_cran()
+  testthat::skip_on_ci()
+  skip_if_no_torch()
+
+  Y = rbinom(50, 1, 0.5)
+  X = rnorm(50)
+  m = dnn(Y~., data = data.frame(Y = Y, X = X), loss = "binomial", epochs = 2L, verbose = FALSE, plot = FALSE)
+  testthat::expect_equal( !!m$base_loss, !!(-sum(dbinom(Y, 1, (mean(Y)), log = TRUE)/50)), tolerance = 0.01)
+
+  Y = rpois(50, 5)
+  X = rnorm(50)
+  m = dnn(Y~., data = data.frame(Y = Y, X = X), loss = "poisson", epochs = 2L, verbose = FALSE, plot = FALSE)
+  testthat::expect_equal( !!m$base_loss, !!(-sum(dpois(Y, (mean(Y)), log = TRUE)/50)), tolerance = 0.01)
+
+  Y = rnorm(50, 5)
+  X = rnorm(50)
+  m = dnn(Y~., data = data.frame(Y = Y, X = X), loss = "mse", epochs = 2L, verbose = FALSE, plot = FALSE)
+  testthat::expect_equal( !!m$base_loss, !!mean((Y - mean(Y))**2 ), tolerance = 0.01)
+
+  Y = rbinom(50, 2, 0.5)
+  X = rnorm(50)
+  m = dnn(Y~., data = data.frame(Y = as.factor(Y+1), X = X), loss = "softmax", epochs = 2L, verbose = FALSE, plot = FALSE)
+  pred = log(matrix(table(as.factor(Y+1))/sum(table(as.factor(Y+1))), 50, 3, byrow = TRUE)) + log(3)
+  loss = as.numeric(torch::nnf_cross_entropy(pred, torch::torch_tensor(Y+1, dtype = torch::torch_long())))
+  testthat::expect_equal( !!m$base_loss, !!loss , tolerance = 0.01)
+
+  })
+
+
 
 
 
